/*
   DEMO WANDER
*/

#include <stdio.h>
#include <libplayerc/playerc.h>

int main(int argc, const char **argv)
{
  int i;
  playerc_client_t *client;
  playerc_position2d_t *position2d;
  playerc_laser_t *laser;

  // Create a client and connect it to the server.
  client = playerc_client_create(NULL, "localhost", 6665);
  if (0 != playerc_client_connect(client))
    return -1;

  // Create and subscribe to a position2d device.
  position2d = playerc_position2d_create(client, 0);
  if (playerc_position2d_subscribe(position2d, PLAYER_OPEN_MODE))
    return -1;

  laser = playerc_laser_create(client, 0);
  if (playerc_laser_subscribe(laser, PLAYERC_OPEN_MODE))
    return -1;

  // Make the robot spin!
  if (0 != playerc_position2d_set_cmd_vel(position2d, 0, 0, DTOR(40.0), 1))
    return -1;

  // REAL Robot: Execute serveral reads before start using the robot...
  playerc_client_read(client);
  playerc_client_read(client);
  playerc_client_read(client);
  playerc_client_read(client);
  playerc_client_read(client);
  
    // go into read-think-act loop
    for(;;)
    {
      double newspeed = 0;
      double newturnrate = 0;
      double minR;
      double minL;

      // this blocks until new data comes; 10Hz by default
      playerc_client_read(client);

      minR = minL = 9999;
      for (i=0; i < 180; i++)
      {
          if (laser->scan[i][0] < minR) minR = laser->scan[i][0];
          if (laser->scan[i+180][0] < minL) minL = laser->scan[i+180][0];
      }

      // laser avoid (stolen from esben's java example)
      // std::cout << "minR: " << minR
      //          << "minL: " << minL
      //          << std::endl;

      double l = (1e5*minR)/500-100;
      double r = (1e5*minL)/500-100;

      if (l > 100)
        l = 100;
      if (r > 100)
        r = 100;

      newspeed = (r+l)/1e3;

      newturnrate = (r-l);
      if (newturnrate < -40.0) newturnrate = -40.0;
      if (newturnrate >  40.0) newturnrate =  40.0;

      // std::cout << "speed: " << newspeed
      //          << "turn: " << newturnrate
      //          << std::endl;

      // write commands to robot
      // pp.SetSpeed(newspeed, newturnrate);
      playerc_position2d_set_cmd_vel(position2d, newspeed, 0, DTOR(newturnrate), 1);
    }

  // Shutdown
  playerc_position2d_unsubscribe(position2d);
  playerc_position2d_destroy(position2d);
  playerc_client_disconnect(client);
  playerc_client_destroy(client);

  return 0;
}
